@@ -31,9 +31,8 @@
 
 #include "sysincl.h"
 
-/* Read the system clock.  This is analogous to gettimeofday(),
-   but with the timezone information ignored */
-extern void LCL_ReadRawTime(struct timeval *);
+/* Read the system clock */
+extern void LCL_ReadRawTime(struct timespec *ts);
 
 /* Read the system clock, corrected according to all accumulated
    drifts and uncompensated offsets.
@@ -44,15 +43,15 @@ extern void LCL_ReadRawTime(struct timeval *);
    adjtime()-like interface to correct offsets, and to adjust the
    frequency), we must correct the raw time to get this value */
 
-extern void LCL_ReadCookedTime(struct timeval *t, double *err);
+extern void LCL_ReadCookedTime(struct timespec *ts, double *err);
 
 /* Convert raw time to cooked. */
-extern void LCL_CookTime(struct timeval *raw, struct timeval *cooked, double *err);
+extern void LCL_CookTime(struct timespec *raw, struct timespec *cooked, double *err);
 
 /* Read the current offset between the system clock and true time
    (i.e. 'cooked' - 'raw') (in seconds). */
 
-extern void LCL_GetOffsetCorrection(struct timeval *raw, double *correction, double *err);
+extern void LCL_GetOffsetCorrection(struct timespec *raw, double *correction, double *err);
 
 /* Type of routines that may be invoked as callbacks when there is a
    change to the frequency or offset.
@@ -79,7 +78,7 @@ typedef enum {
 } LCL_ChangeType;
 
 typedef void (*LCL_ParameterChangeHandler)
-     (struct timeval *raw, struct timeval *cooked,
+     (struct timespec *raw, struct timespec *cooked,
       double dfreq,
       double doffset,
       LCL_ChangeType change_type,
@@ -163,7 +162,7 @@ extern int LCL_ApplyStepOffset(double offset);
 
 /* Routine to invoke notify handlers on an unexpected time jump
    in system clock */
-extern void LCL_NotifyExternalTimeStep(struct timeval *raw, struct timeval *cooked,
+extern void LCL_NotifyExternalTimeStep(struct timespec *raw, struct timespec *cooked,
     double offset, double dispersion);
 
 /* Routine to invoke notify handlers on leap second when the system clock
@@ -202,10 +201,11 @@ extern int LCL_MakeStep(void);
    does something */
 extern int LCL_CanSystemLeap(void);
 
-/* Routine to set the system clock to correct itself for a leap second if
-   supported.  Leap second will be inserted at the end of the day if the
-   argument is positive, deleted if negative, and zero resets the setting. */
-extern void LCL_SetSystemLeap(int leap);
+/* Routine to set the system clock to correct itself for a leap second and also
+   set its TAI-UTC offset.  If supported, leap second will be inserted at the
+   end of the day if the argument is positive, deleted if negative, and zero
+   resets the setting. */
+extern void LCL_SetSystemLeap(int leap, int tai_offset);
 
 /* Routine to set a frequency correction (in ppm) that should be applied
    to local clock to compensate for temperature changes.  A positive
