@@ -60,34 +60,44 @@ static void
 accrue_offset(double offset, double corr_rate)
 {
   struct timeval newadj, oldadj;
+  double doldadj;
 
   UTI_DoubleToTimeval(-offset, &newadj);
 
   if (PRV_AdjustTime(&newadj, &oldadj) < 0)
-    LOG_FATAL(LOGF_SysNetBSD, "adjtime() failed");
+    LOG_FATAL("adjtime() failed");
 
   /* Add the old remaining adjustment if not zero */
-  UTI_TimevalToDouble(&oldadj, &offset);
-  if (offset != 0.0) {
-    UTI_AddDoubleToTimeval(&newadj, offset, &newadj);
+  doldadj = UTI_TimevalToDouble(&oldadj);
+  if (doldadj != 0.0) {
+    UTI_DoubleToTimeval(-offset + doldadj, &newadj);
     if (PRV_AdjustTime(&newadj, NULL) < 0)
-      LOG_FATAL(LOGF_SysNetBSD, "adjtime() failed");
+      LOG_FATAL("adjtime() failed");
   }
 }
 
 /* ================================================== */
 
 static void
-get_offset_correction(struct timeval *raw,
+get_offset_correction(struct timespec *raw,
                       double *corr, double *err)
 {
   struct timeval remadj;
   double adjustment_remaining;
+#ifdef MACOSX
+  struct timeval tv = {0, 0};
 
+  if (PRV_AdjustTime(&tv, &remadj) < 0)
+    LOG_FATAL("adjtime() failed");
+
+  if (PRV_AdjustTime(&remadj, NULL) < 0)
+    LOG_FATAL("adjtime() failed");
+#else
   if (PRV_AdjustTime(NULL, &remadj) < 0)
-    LOG_FATAL(LOGF_SysNetBSD, "adjtime() failed");
+    LOG_FATAL("adjtime() failed");
+#endif
 
-  UTI_TimevalToDouble(&remadj, &adjustment_remaining);
+  adjustment_remaining = UTI_TimevalToDouble(&remadj);
 
   *corr = adjustment_remaining;
   if (err) {
@@ -137,7 +147,7 @@ SYS_NetBSD_DropRoot(uid_t uid, gid_t gid)
   /* Check if we have write access to /dev/clockctl */
   fd = open("/dev/clockctl", O_WRONLY);
   if (fd < 0)
-    LOG_FATAL(LOGF_SysNetBSD, "Can't write to /dev/clockctl");
+    LOG_FATAL("Can't write to /dev/clockctl");
   close(fd);
 #endif
 }
